package Simple::Filter::Macro;

# Set the VERSION.
$VERSION = "0.02";

# Load the Perl pragmas.
use strict;
use warnings;

# Outer filter starts here.
use Filter::Simple::Compile sub {
    # Remove the package terminator from the package content.
    $_ =~ s/1;\s//g;
    # Create the changed script content.
    $_ = sprintf(q(
        # Inner filter starts here.
        use Filter::Simple::Compile sub {
            # Concatenate content from package and script. 
            $_ = join("\n",
                '# Line ' . (__LINE__+1) . ' ' .__FILE__,
                "%s",
                '# Line %s %s',
                $_,
            );
        };
        # Inner filter ends here.
        1;
    ), $_, (caller(6))[2]+1, (caller(6))[1]);
};
# Outer filter ends here.
1;

=head1 NAME

Simple::Filter::Macro - Perl extension for expanding source code inline in a script or module. 

=head1 SYNOPSIS

In script or module:

  use ExamplePackage::Modules;

In module F<Modules.pm> of a package C<ExamplePackage>:

  package ExamplePackage::Modules;
  # Set the VERSION number.
  $VERSION = '0.01';
  # Use the magic module.
  use Simple::Filter::Macro; # <-- The magic is found here.
  # The lines below will be expanded into the caller's code.
  use strict;
  use warnings;
  use diagnostics;
  use sigtrap; 
  use ExamplePackage::ReadFile;
  use ExamplePackage::ModifyFile;
  use ExamplePackage::WriteFile;
  use re 'debug'
  no utf8;
  # Add as much use statements as you like.
  # Package terminator 1; will not be written to caller's code.
  1;   

=head1 DESCRIPTION

The module is expanding source code inline in a script or a module. This
statement is valid, when the use statement is reachable while compilation.
It will not work in a if statement or code block.

One use case can be find in the SYNOPSIS. It is common practice to load
modules using the C<use> statement while compilation. Subroutine imports of
modules from C<ExamplePackage> takes effect in that module but not in the
calling script. The same thing is valid for Perl pragmas. They take effect in
the script or in the module not in the calling script or module.

It's a kind of magic when it is possible to change the source code while
compilation. That's happens here. The package defined module as well as the
script of an user creates a compiled file with the extensions .pmc and .plc.         

=head1 METHOD

None

=head1 EXPORT

None

=head1 PROGRAMMING BACKGROUND

A outer filter is applied. In this loop, the content of the given module is
modified in a way, that comments are removed and the module terminator 1; is 
is stripped.  

Then a inner filter is applied. In the inner filter the module content is
concatenated to the content of the given script. Both is written to the file
with the head created applying the filter procedure.

=head1 NOTES

Use

  use Simple::Filter::MacroLight; # <-- The magic is found here.
 
for a prefiltered C<.plc> file. C<Simple::Filter::MacroLight> is removing
comments from the script content before the complete content is written to
a file.

=head1 KNOWN ISSUES

Running a script using Simple::Filter::Macro
 
  perl test_script.pl 

will create an error message on the second run. If the script runs using

  ./test_script.pl

no error occurs.

Nevertheless 

  perl test_script.plc

is all the time working. This has to checked.

=head1 TO-DO

Improvement of the documentation.

=head1 MOTIVATION

I was looking for a module that would allow me to merge a number of C<use>
pragma declarations in the module and replace them with a single C<use>
pragma declaration. The module C<Filter::Macro> that I found contains the
right approaches for this, but is not executable under Perl v5.30 how I found
out. After analysing the source code, I decided to create a new module based
on the aforementioned approaches. The problem in the source code As I figured
out is the use of the Perl command C<quotemeta>. Since Perl v5.16 the use of
C<quotemeta> is obsolet as I suppose. The first attempts with basic changes in
the source code were not without errors. In the meantime, the Perl code works
as expected. The first two methods of this module are the result of my efforts.  

=head1 EXPORT

None

=head1 SEE ALSO

L<Filter::Simple::Compile|https://metacpan.org/pod/Filter::Simple::Compile/>

L<Filter::Macro|https://metacpan.org/pod/Filter::Macro/>

=head1 CREDITS

Special thanks go to Audrey Tang, who wrote the C<Filter::Macro> module, which
contains the crucial approaches to implementing the present package and its
modules and methods. 

=head1 AUTHOR

Dr. Peter Netz, E<lt>ztenretep@cpan.orgE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2022 by Dr. Peter Netz

This library is free software; you can redistribute it and/or modify it
under the same terms of The MIT License. For more details, see the full
text of the license in the attached file LICENSE in the main module folder.
This program is distributed in the hope that it will be useful, but without
any warranty; without even the implied warranty of merchantability or fitness
for a particular purpose.

=cut
